@@ -4,17 +4,10 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE GADTs #-}
-#if __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
 {-# LANGUAGE DeriveGeneric #-}
-#endif
-#if __GLASGOW_HASKELL__ >= 707
 {-# LANGUAGE DataKinds #-}
-#endif
-
-#if __GLASGOW_HASKELL__ >= 800
 {-# LANGUAGE DeriveLift #-}
-#endif
 
 #ifndef MIN_VERSION_vector
 #define MIN_VERSION_vector(x,y,z) 1
@@ -65,68 +58,53 @@ import Control.DeepSeq (NFData(rnf))
 import Control.Monad (liftM)
 import Control.Monad.Fix
 import Control.Monad.Zip
-import Control.Lens hiding (index, (<.>))
+import Control.Lens as Lens hiding (index, (<.>))
 import Data.Binary as Binary
 import Data.Bytes.Serial
 import Data.Distributive
 import Data.Foldable as Foldable
+import qualified Data.Foldable.WithIndex as WithIndex
 import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
+import qualified Data.Functor.WithIndex as WithIndex
 import Data.Hashable
 import Data.Semigroup
 import Data.Semigroup.Foldable
 import Data.Serialize as Cereal
-#if __GLASGOW_HASKELL__ >= 707
+import qualified Data.Traversable.WithIndex as WithIndex
 import qualified Data.Vector as V
-#endif
 import qualified Data.Vector.Generic.Mutable as M
 import qualified Data.Vector.Generic as G
 import qualified Data.Vector.Unboxed.Base as U
 import Foreign.Ptr (castPtr)
 import Foreign.Storable (Storable(..))
 import GHC.Arr (Ix(..))
-#if __GLASGOW_HASKELL__ >= 702
-import GHC.Generics (Generic)
-#endif
-#if __GLASGOW_HASKELL__ >= 706
-import GHC.Generics (Generic1)
-#endif
-#if __GLASGOW_HASKELL__ >= 800
+import GHC.Generics (Generic, Generic1)
+#if defined(MIN_VERSION_template_haskell)
 import Language.Haskell.TH.Syntax (Lift)
 #endif
 import Linear.Epsilon
 import Linear.Metric
-#if __GLASGOW_HASKELL__ >= 707
 import Linear.V
-#endif
 import Linear.V2
 import Linear.V3
 import Linear.V4
 import Linear.Vector
-import System.Random
-
-{-# ANN module "HLint: ignore Reduce duplication" #-}
+import System.Random (Random(..))
 
 -- | Plücker coordinates for lines in a 3-dimensional space.
 data Plucker a = Plucker !a !a !a !a !a !a deriving (Eq,Ord,Show,Read
-#if __GLASGOW_HASKELL__ >= 702
-                                                    ,Generic
-#endif
-#if __GLASGOW_HASKELL__ >= 706
-                                                    ,Generic1
-#endif
-#if __GLASGOW_HASKELL__ >= 800
+                                                    ,Generic,Generic1
+#if defined(MIN_VERSION_template_haskell)
                                                     ,Lift
 #endif
                                                     )
 
-#if __GLASGOW_HASKELL__ >= 707
 instance Finite Plucker where
   type Size Plucker = 6
   toV (Plucker a b c d e f) = V (V.fromListN 6 [a,b,c,d,e,f])
   fromV (V v) = Plucker (v V.! 0) (v V.! 1) (v V.! 2) (v V.! 3) (v V.! 4) (v V.! 5)
-#endif
 
 instance Random a => Random (Plucker a) where
   random g = case random g of
@@ -179,8 +157,10 @@ instance Bind Plucker where
   {-# INLINE (>>-) #-}
 
 instance Monad Plucker where
+#if !(MIN_VERSION_base(4,11,0))
   return a = Plucker a a a a a a
   {-# INLINE return #-}
+#endif
   Plucker a b c d e f >>= g = Plucker a' b' c' d' e' f' where
     Plucker a' _ _ _ _ _ = g a
     Plucker _ b' _ _ _ _ = g b
@@ -210,10 +190,8 @@ instance Foldable Plucker where
   foldMap g (Plucker a b c d e f) =
     g a `mappend` g b `mappend` g c `mappend` g d `mappend` g e `mappend` g f
   {-# INLINE foldMap #-}
-#if __GLASGOW_HASKELL__ >= 710
   null _ = False
   length _ =  6
-#endif
 
 instance Traversable Plucker where
   traverse g (Plucker a b c d e f) =
@@ -433,25 +411,31 @@ e23 = E p23
 e31 = E p31
 e12 = E p12
 
-instance FunctorWithIndex (E Plucker) Plucker where
+instance WithIndex.FunctorWithIndex (E Plucker) Plucker where
   imap f (Plucker a b c d e g) = Plucker (f e01 a) (f e02 b) (f e03 c) (f e23 d) (f e31 e) (f e12 g)
   {-# INLINE imap #-}
 
-instance FoldableWithIndex (E Plucker) Plucker where
+instance WithIndex.FoldableWithIndex (E Plucker) Plucker where
   ifoldMap f (Plucker a b c d e g) = f e01 a `mappend` f e02 b `mappend` f e03 c
                            `mappend` f e23 d `mappend` f e31 e `mappend` f e12 g
   {-# INLINE ifoldMap #-}
 
-instance TraversableWithIndex (E Plucker) Plucker where
+instance WithIndex.TraversableWithIndex (E Plucker) Plucker where
   itraverse f (Plucker a b c d e g) = Plucker <$> f e01 a <*> f e02 b <*> f e03 c
                                               <*> f e23 d <*> f e31 e <*> f e12 g
   {-# INLINE itraverse #-}
 
+#if !MIN_VERSION_lens(5,0,0)
+instance Lens.FunctorWithIndex     (E Plucker) Plucker where imap      = WithIndex.imap
+instance Lens.FoldableWithIndex    (E Plucker) Plucker where ifoldMap  = WithIndex.ifoldMap
+instance Lens.TraversableWithIndex (E Plucker) Plucker where itraverse = WithIndex.itraverse
+#endif
+
 type instance Index (Plucker a) = E Plucker
 type instance IxValue (Plucker a) = a
 
 instance Ixed (Plucker a) where
-  ix = el
+  ix i = el i
   {-# INLINE ix #-}
 
 instance Each (Plucker a) (Plucker b) a b where
@@ -494,11 +478,7 @@ data LinePass = Coplanar
               | Counterclockwise
               -- ^ The lines pass each other counterclockwise
               -- (left-handed screw).
-                deriving (Eq, Show
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
-                         ,Generic
-#endif
-                         )
+                deriving (Eq, Show,Generic)
 
 -- | Check how two lines pass each other. @passes l1 l2@ describes
 -- @l2@ when looking down @l1@.
@@ -527,20 +507,49 @@ toUV (Plucker a b c d e f) = V2 (V3 a b c) (V3 d e f)
 -- | Checks if two lines coincide in space. In other words, undirected equality.
 coincides :: (Epsilon a, Fractional a) => Plucker a -> Plucker a -> Bool
 coincides p1 p2 = Foldable.all nearZero $ (s *^ p2) - p1
-  where s = maybe 1 getFirst . getOption . fold $ saveDiv <$> p1 <*> p2
-        saveDiv x y | nearZero y = Option Nothing
-                    | otherwise  = Option . Just $ First (x / y)
+  where s = maybe 1 getFirst . getOptionCompat . fold $ saveDiv <$> p1 <*> p2
+        saveDiv x y | nearZero y = optionCompat Nothing
+                    | otherwise  = optionCompat . Just $ First (x / y)
 {-# INLINABLE coincides #-}
 
 -- | Checks if two lines coincide in space, and have the same
 -- orientation.
 coincides' :: (Epsilon a, Fractional a, Ord a) => Plucker a -> Plucker a -> Bool
 coincides' p1 p2 = Foldable.all nearZero ((s *^ p2) - p1) && s > 0
-  where s = maybe 1 getFirst . getOption . fold $ saveDiv <$> p1 <*> p2
-        saveDiv x y | nearZero y = Option Nothing
-                    | otherwise  = Option . Just $ First (x / y)
+  where s = maybe 1 getFirst . getOptionCompat . fold $ saveDiv <$> p1 <*> p2
+        saveDiv x y | nearZero y = optionCompat Nothing
+                    | otherwise  = optionCompat . Just $ First (x / y)
 {-# INLINABLE coincides' #-}
 
+-- The coincides and coincides' functions above require the use of a Maybe type
+-- with the following Monoid instance:
+--
+--   instance Semigroup a => Monoid (Maybe a) where ...
+--
+-- Unfortunately, Maybe has only had such an instance since base-4.11. Prior
+-- to that, its Monoid instance had an instance context of Monoid a, which is
+-- too strong. To compensate, we use CPP to define an OptionCompat type
+-- synonym, which is an alias for Maybe on recent versions of base and an alias
+-- for Data.Semigroup.Option on older versions of base. We don't want to use
+-- Option on recent versions of base, as it is deprecated.
+#if MIN_VERSION_base(4,11,0)
+type OptionCompat = Maybe
+
+optionCompat :: Maybe a -> OptionCompat a
+optionCompat = id
+
+getOptionCompat :: OptionCompat a -> Maybe a
+getOptionCompat = id
+#else
+type OptionCompat = Option
+
+optionCompat :: Maybe a -> OptionCompat a
+optionCompat = Option
+
+getOptionCompat :: OptionCompat a -> Maybe a
+getOptionCompat = getOption
+#endif
+
 -- | The minimum squared distance of a line from the origin.
 quadranceToOrigin :: Fractional a => Plucker a -> a
 quadranceToOrigin p = (v `dot` v) / (u `dot` u)
@@ -590,9 +599,7 @@ instance U.Unbox a => M.MVector U.MVector (Plucker a) where
        M.basicUnsafeWrite a (o+3) w
        M.basicUnsafeWrite a (o+4) v
        M.basicUnsafeWrite a (o+5) u
-#if MIN_VERSION_vector(0,11,0)
   basicInitialize (MV_Plucker _ v) = M.basicInitialize v
-#endif
 
 instance U.Unbox a => G.Vector U.Vector (Plucker a) where
   basicUnsafeFreeze (MV_Plucker n v) = liftM ( V_Plucker n) (G.basicUnsafeFreeze v)
@@ -640,8 +647,6 @@ instance Serialize a => Serialize (Plucker a) where
   put = serializeWith Cereal.put
   get = deserializeWith Cereal.get
 
-
-#if (MIN_VERSION_transformers(0,5,0)) || !(MIN_VERSION_transformers(0,4,0))
 instance Eq1 Plucker where
   liftEq k (Plucker a1 b1 c1 d1 e1 f1)
            (Plucker a2 b2 c2 d2 e2 f2)
@@ -664,12 +669,6 @@ instance Read1 Plucker where
 instance Show1 Plucker where
   liftShowsPrec k _ z (Plucker a b c d e f) = showParen (z > 10) $
      showString "Plucker " . k 11 a . showChar ' ' . k 11 b . showChar ' ' . k 11 c . showChar ' ' . k 11 d . showChar ' ' . k 11 e . showChar ' ' . k 11 f
-#else
-instance Eq1 Plucker where eq1 = (==)
-instance Ord1 Plucker where compare1 = compare
-instance Show1 Plucker where showsPrec1 = showsPrec
-instance Read1 Plucker where readsPrec1 = readsPrec
-#endif
 
 instance Field1 (Plucker a) (Plucker a) a a where
   _1 f (Plucker x y z u v w) = f x <&> \x' -> Plucker x' y z u v w
